@@ -1,6 +1,8 @@
 use swc_atoms::JsWord;
 use swc_common::DUMMY_SP;
 use swc_ecma_ast::*;
+use swc_ecma_transforms_base::perf::Parallel;
+use swc_ecma_transforms_macros::parallel;
 use swc_ecma_utils::{quote_ident, ExprFactory};
 use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut, VisitMutWith};
 
@@ -22,9 +24,17 @@ pub fn sticky_regex() -> impl 'static + Fold + VisitMut {
     as_folder(StickyRegex)
 }
 
-#[derive(Clone, Copy)]
 struct StickyRegex;
 
+impl Parallel for StickyRegex {
+    fn merge(&mut self, _: Self) {}
+
+    fn create(&self) -> Self {
+        StickyRegex
+    }
+}
+
+#[parallel]
 impl VisitMut for StickyRegex {
     noop_visit_mut_type!();
 
